"""Class implementations for the millisecond-related mix-in.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class MillisecondMixIn(
    VariableNameMixIn, VariableNameSuffixAttrOrVarMixIn, RevertMixIn
):
    _initial_millisecond: Union[int, Int]
    _millisecond: Int

    @final
    @add_debug_info_setting(module_name=__name__)
    @arg_validation_decos.is_millisecond_int(arg_position_index=1)
    def _set_init_millisecond_value(self, *, millisecond: Union[int, Int]) -> None:
        """
        Set an initial millisecond value.

        Parameters
        ----------
        millisecond : Union[int, Int]
            A millisecond value to set.
        """
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_int_from_builtin_val,
        )

        self._initial_millisecond = millisecond
        suffix: str = self._get_attr_or_variable_name_suffix(
            value_identifier="millisecond"
        )
        self._millisecond = get_copied_int_from_builtin_val(
            integer=millisecond,
            variable_name_suffix=suffix,
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _get_init_millisecond_argument_expression(self) -> str:
        """
        Get an initial millisecond argument expression string.

        Returns
        -------
        expression : str
            A created expression string.
        """
        if isinstance(self._initial_millisecond, Int):
            return f", {self._initial_millisecond.variable_name}"
        return f", {self._millisecond._value}"

    @property
    @add_debug_info_setting(module_name=__name__)
    def millisecond(self) -> Int:
        """
        Get a current millisecond value.

        Returns
        -------
        millisecond : Int
            A current millisecond value.

        References
        ----------
        - DateTime class millisecond property
            - https://simon-ritchie.github.io/apysc/en/datetime_millisecond.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> datetime_: ap.DateTime = ap.DateTime(
        ...     year=2022, month=12, day=1, millisecond=500
        ... )
        >>> datetime_.millisecond
        Int(500)
        >>> datetime_.millisecond = ap.Int(300)
        >>> datetime_.millisecond
        Int(300)
        """
        copied_millisecond_val: Int = self._millisecond.copy()
        self._append_millisecond_getter_expression(
            millisecond_val=copied_millisecond_val
        )
        return copied_millisecond_val

    @millisecond.setter
    @arg_validation_decos.is_millisecond_int(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def millisecond(self, value: Int) -> None:
        """
        Set a millisecond value.

        Parameters
        ----------
        value : Int
            A millisecond value to set.

        References
        ----------
        - DateTime class millisecond property
            - https://simon-ritchie.github.io/apysc/en/datetime_millisecond.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> datetime_: ap.DateTime = ap.DateTime(
        ...     year=2022, month=12, day=1, millisecond=500
        ... )
        >>> datetime_.millisecond
        Int(500)
        >>> datetime_.millisecond = ap.Int(300)
        >>> datetime_.millisecond
        Int(300)
        """
        self._millisecond = value.copy()
        self._append_millisecond_setter_expression(millisecond_val=value)

    @final
    @arg_validation_decos.is_millisecond_int(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def _append_millisecond_getter_expression(self, *, millisecond_val: Int) -> None:
        """
        Append a millisecond's getter expression string.

        Parameters
        ----------
        millisecond_val : Int
            A millisecond value to use in an expression.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{millisecond_val.variable_name} = {self.variable_name}.getMilliseconds();"
        )
        expression_data_util.append_js_expression(expression)

    def _append_millisecond_setter_expression(self, *, millisecond_val: Int) -> None:
        """
        Append a millisecond's setter expression string.

        Parameters
        ----------
        millisecond_val : Int
            A millisecond value to use in an expression.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{self.variable_name}.setMilliseconds({millisecond_val.variable_name});"
        )
        expression_data_util.append_js_expression(expression=expression)

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._millisecond._run_all_make_snapshot_methods(snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._millisecond._run_all_revert_methods(snapshot_name=snapshot_name)



"""This module is for the translation mapping data of the
following document:

Document file: timedelta.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# TimeDelta class": "# TimeDelta クラス",
    ##################################################
    "This page explains the `TimeDelta` class.": "このページでは`TimeDelta`クラスについて説明します。",
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `TimeDelta` class treats a time delta between two `DateTime` instances.": "`TimeDelta`クラスは2つの`DateTime`クラスのインスタンス間の時間差を扱います。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Subtraction between two `DateTime` instances returns this class's instance.": "2つの`DateTime`クラスのインスタンス間の減算はこのクラスのインスタンスを返却します。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndatetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\ndatetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)\ntimedelta_: ap.TimeDelta = datetime_1 - datetime_2\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndatetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\ndatetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)\ntimedelta_: ap.TimeDelta = datetime_1 - datetime_2\n```",  # noqa
    ##################################################
    "A `TimeDelta` instance has each interface, such as the `days`' property or `total_seconds`' method, as follows:": "`TimeDelta`クラスのインスタンスは以下のように`days`属性や`total_seconds`メソッドなどの各インターフェイスを持っています:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndatetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\ndatetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)\ntimedelta_: ap.TimeDelta = datetime_1 - datetime_2\ndays: ap.Int = timedelta_.days\nassert days == 2\ntotal_seconds: ap.Number = timedelta_.total_seconds()\nassert total_seconds == 60 * 60 * 24 * 2\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndatetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\ndatetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)\ntimedelta_: ap.TimeDelta = datetime_1 - datetime_2\ndays: ap.Int = timedelta_.days\nassert days == 2\ntotal_seconds: ap.Number = timedelta_.total_seconds()\nassert total_seconds == 60 * 60 * 24 * 2\n```",  # noqa
    ##################################################
    "## days property API": "## days 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get days in the duration.<hr>": "時間の間隔値の日数を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `days`: Int": "- `days`: Int",
    ##################################################
    "  - Days value. This interface ignores a fraction.": "  - 日数値。小数点数の分は無視されます。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> datetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\n>>> datetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)\n>>> timedelta_: ap.TimeDelta = datetime_1 - datetime_2\n>>> timedelta_.days\nInt(2)\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> datetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\n>>> datetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)\n>>> timedelta_: ap.TimeDelta = datetime_1 - datetime_2\n>>> timedelta_.days\nInt(2)\n```",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [TimeDelta class days interface](https://simon-ritchie.github.io/apysc/en/timedelta_days.html)": "- [TimeDelta クラスの days インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_timedelta_days.html)",  # noqa
    ##################################################
    "## total_seconds method API": "## total_seconds メソッドのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the total seconds in the duration.<hr>": "時間の間隔値の合計秒数を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `total_seconds`: Number": "- `total_seconds`: Number",
    ##################################################
    "  - Total seconds in the duration.": "  - 時間の間隔値の合計秒数。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> datetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\n>>> datetime_2: ap.DateTime = ap.DateTime(2022, 12, 6)\n>>> timedelta_: ap.TimeDelta = datetime_1 - datetime_2\n>>> timedelta_.total_seconds()\nNumber(86400.0)\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> datetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)\n>>> datetime_2: ap.DateTime = ap.DateTime(2022, 12, 6)\n>>> timedelta_: ap.TimeDelta = datetime_1 - datetime_2\n>>> timedelta_.total_seconds()\nNumber(86400.0)\n```",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [TimeDelta class total_seconds interface](https://simon-ritchie.github.io/apysc/en/timedelta_total_seconds.html)": "- [TimeDelta クラスの total_seconds インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_timedelta_total_seconds.html)",  # noqa
}



"""The class implementation for the `select_all` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSelectAllIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `select_all` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M3 5h2V3c-1.1 0-2 .9-2 2zm0 8h2v-2H3v2zm4 8h2v-2H7v2zM3 9h2V7H3v2zm10-6h-2v2h2V3zm6 0v2h2c0-1.1-.9-2-2-2zM5 21v-2H3c0 1.1.9 2 2 2zm-2-4h2v-2H3v2zM9 3H7v2h2V3zm2 18h2v-2h-2v2zm8-8h2v-2h-2v2zm0 8c1.1 0 2-.9 2-2h-2v2zm0-12h2V7h-2v2zm0 8h2v-2h-2v2zm-4 4h2v-2h-2v2zm0-16h2V3h-2v2zM7 17h10V7H7v10zm2-8h6v6H9V9z"/></svg>'  # noqa



"""The mix-in class for the `get_colors_members` method.
"""

from typing import List
from typing import Tuple

from apysc._color.color import Color
from apysc._html.debug_mode import add_debug_info_setting


class GetColorsMenmbersMixIn:
    @classmethod
    @add_debug_info_setting(module_name=__name__)
    def get_colors_members(cls) -> List[Tuple[str, Color]]:
        """
        Get colors' members of this instance.

        Returns
        -------
        members : List[Tuple[str, Color]]
            A colors' members of this instance.
            A tuple's first element is a constant name,
            and second is a color instance.
        """
        members: List[Tuple[str, Color]] = []
        for member_name, member_value in cls.__dict__.items():
            if not isinstance(member_value, Color):
                continue
            members.append((member_name, member_value))
        return members



"""The class implementation for the outlined `clear` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialClearOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `clear` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12 19 6.41z"/></svg>'  # noqa



"""The class implementation for the `shopping_bag` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialShoppingBagIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `shopping_bag` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M18,6h-2c0-2.21-1.79-4-4-4S8,3.79,8,6H6C4.9,6,4,6.9,4,8v12c0,1.1,0.9,2,2,2h12c1.1,0,2-0.9,2-2V8C20,6.9,19.1,6,18,6z M10,10c0,0.55-0.45,1-1,1s-1-0.45-1-1V8h2V10z M12,4c1.1,0,2,0.9,2,2h-4C10,4.9,10.9,4,12,4z M16,10c0,0.55-0.45,1-1,1 s-1-0.45-1-1V8h2V10z"/></g></svg>'  # noqa



"""The SVG icon class implementation.
"""

from typing import Optional
from typing import Union

from apysc._color.color import Color
from apysc._color.colors import Colors
from apysc._converter.to_apysc_val_from_builtin import get_copied_int_from_builtin_val
from apysc._display.add_to_parent_mixin import AddToParentMixIn
from apysc._display.append_fill_alpha_attr_expression_mixin import (
    AppendFillAlphaAttrExpressionMixIn,
)
from apysc._display.append_fill_color_expression_mixin import (
    AppendFillColorAttrExpressionMixIn,
)
from apysc._display.append_x_attr_expression_mixin import AppendXAttrExpressionMixIn
from apysc._display.append_y_attr_expression_mixin import AppendYAttrExpressionMixIn
from apysc._display.child_mixin import ChildMixIn
from apysc._display.css_mixin import CssMixIn
from apysc._display.display_object import DisplayObject
from apysc._display.fill_alpha_mixin import FillAlphaMixIn
from apysc._display.fill_color_mixin import FillColorMixIn
from apysc._display.get_bounds_mixin import GetBoundsMixIn
from apysc._display.height_mixin import HeightMixIn
from apysc._display.set_overflow_visible_setting_mixin import (
    SetOverflowVisibleSettingMixIn,
)
from apysc._display.use_hand_cursor_mixin import UseHandCursorMixIn
from apysc._display.width_mixin import WidthMixIn
from apysc._display.x_mixin import XMixIn
from apysc._display.y_mixin import YMixIn
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class SvgIcon(
    XMixIn,
    AppendXAttrExpressionMixIn,
    YMixIn,
    AppendYAttrExpressionMixIn,
    FillColorMixIn,
    AppendFillColorAttrExpressionMixIn,
    FillAlphaMixIn,
    AppendFillAlphaAttrExpressionMixIn,
    WidthMixIn,
    HeightMixIn,
    CssMixIn,
    DisplayObject,
    AddToParentMixIn,
    SetOverflowVisibleSettingMixIn,
    GetBoundsMixIn,
    UseHandCursorMixIn,
    VariableNameSuffixMixIn,
):
    _svg_icon_html: str

    # svg_icon_html
    @arg_validation_decos.is_builtin_string(arg_position_index=1, optional=False)
    # x
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    # y
    @arg_validation_decos.is_num(arg_position_index=3, optional=False)
    # size
    @arg_validation_decos.is_integer(arg_position_index=4, optional=False)
    # fill_color
    @arg_validation_decos.is_color(arg_position_index=5, optional=False)
    # fill_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=6, optional=False)
    # parent
    @arg_validation_decos.is_display_object_container(
        arg_position_index=7, optional=True
    )
    # variable_name_suffix
    @arg_validation_decos.is_builtin_string(arg_position_index=8, optional=False)
    def __init__(
        self,
        *,
        svg_icon_html: str,
        x: Union[float, Number] = 0.0,
        y: Union[float, Number] = 0.0,
        size: Union[int, Int] = 24,
        fill_color: Color = Colors.GRAY_666666,
        fill_alpha: Union[float, Number] = 1.0,
        parent: Optional[ChildMixIn] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        The SVG icon class implementation.

        Parameters
        ----------
        svg_icon_html : str
            An SVG icon html string.
            For example, "<svg xmlns="http://www.w3.org/2000/svg" ...>...</svg>"
        x : float or Number, optional
            X-coordinate of the icon.
        y : float or Number, optional
            Y-coordinate of the icon.
        size : int or Int, default 24
            Size of the icon.
        fill_color : Color, default Colors.GRAY_666666
            Fill-color of the icon.
        fill_alpha : float or Number, default 1.0
            Fill-alpha of the icon.
        parent : ChildMixIn or None, default None
            A parent instance to add this instance.
            If the specified value is None, this interface uses
            a stage instance.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_number_from_builtin_val,
        )
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._variable_name_suffix = variable_name_suffix
        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.SVG_ICON
        )
        self.variable_name = variable_name
        self._svg_icon_html = svg_icon_html.replace("'", '"')
        self._append_constructor_expression()
        super(SvgIcon, self).__init__(variable_name=variable_name)
        self.x = get_copied_number_from_builtin_val(float_or_num=x)
        self.y = get_copied_number_from_builtin_val(float_or_num=y)
        size_: Int = get_copied_int_from_builtin_val(integer=size)
        self.width = size_
        self.height = size_
        self.fill_color = fill_color
        self.fill_alpha = get_copied_number_from_builtin_val(float_or_num=fill_alpha)
        self._set_overflow_visible_setting()
        self._add_to_parent(parent=parent)

    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"var {self.variable_name} = " f"SVG('{self._svg_icon_html}');"
        )
        expression_data_util.append_js_expression(expression=expression)



"""This module is for the translation mapping data of the
following document:

Document file: text_underline.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Text underline property": "# テキストの underline 属性",
    ##################################################
    "This page explains the text-related `underline` property.": "このページではテキスト関係の`underline`属性について説明します。",  # noqa
    ##################################################
    "## What property is this?": "## 属性の概要",
    ##################################################
    "The `underline` property sets or gets the text's underline style setting.": "`underline`属性はテキストの下線のスタイル設定の更新もしくは取得を行います。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter and setter interfaces' type becomes an `ap.Boolean` value.": "getterもしくはsetterの各インターフェイスの値は`ap.Boolean`の型となります。",  # noqa
    ##################################################
    "If you specify the `ap.Boolean(True)` (or `ap.True_`) value, a text displays an underline.": "もし`ap.Boolean(True)`（もしくは`ap.True_`）の値を指定した場合、テキストは下線を表示します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=300,\n    stage_height=195,\n    stage_elem_id="stage",\n)\n\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=250,\n    fill_color=ap.Colors.GRAY_AAAAAA,\n    x=25,\n    y=25,\n)\nmulti_line_text.underline = ap.True_\n\nap.save_overall_html(dest_dir_path="./text_underline_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=300,\n    stage_height=195,\n    stage_elem_id="stage",\n)\n\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=250,\n    fill_color=ap.Colors.GRAY_AAAAAA,\n    x=25,\n    y=25,\n)\nmulti_line_text.underline = ap.True_\n\nap.save_overall_html(dest_dir_path="./text_underline_basic_usage/")\n```',  # noqa
    ##################################################
    "## underline property API": "## underline 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a text underline (`text-decoration: underline`) setting.<hr>": "テキストの下線（`text-decoration: underline`）設定を取得します。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `underline`: Boolean": "- `underline`: Boolean",
    ##################################################
    "  - A text underline setting.": "  - テキストの下線の設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=300,\n...     stage_height=195,\n...     stage_elem_id="stage",\n... )\n>>> multi_line_text: ap.MultiLineText = ap.MultiLineText(\n...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n...     "Ut enim ad minim veniam",\n...     width=250,\n...     fill_color=ap.Colors.GRAY_AAAAAA,\n...     x=25,\n...     y=25,\n... )\n>>> multi_line_text.underline = ap.True_\n>>> multi_line_text.underline\nBoolean(True)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=300,\n...     stage_height=195,\n...     stage_elem_id="stage",\n... )\n>>> multi_line_text: ap.MultiLineText = ap.MultiLineText(\n...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n...     "Ut enim ad minim veniam",\n...     width=250,\n...     fill_color=ap.Colors.GRAY_AAAAAA,\n...     x=25,\n...     y=25,\n... )\n>>> multi_line_text.underline = ap.True_\n>>> multi_line_text.underline\nBoolean(True)\n```',  # noqa
}



"""The class implementation for the outlined `filter_alt` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFilterAltOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `filter_alt` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><path d="M0,0h24 M24,24H0" fill="none"/><path d="M7,6h10l-5.01,6.3L7,6z M4.25,5.61C6.27,8.2,10,13,10,13v6c0,0.55,0.45,1,1,1h2c0.55,0,1-0.45,1-1v-6 c0,0,3.72-4.8,5.74-7.39C20.25,4.95,19.78,4,18.95,4H5.04C4.21,4,3.74,4.95,4.25,5.61z"/><path d="M0,0h24v24H0V0z" fill="none"/></g></svg>'  # noqa



"""The mix-in class implementation for the `axis_label_font_family` value.
"""

from typing import List
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.array import Array
from apysc._type.string import String
from apysc._validation import arg_validation_decos


class AxisLabelFontFamilyMixIn:
    _axis_label_font_family: Optional[Array[String]]

    @final
    @arg_validation_decos.is_builtin_str_list_or_apysc_str_arr(
        arg_position_index=1, optional=True
    )
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_axis_label_font_family(
        self,
        *,
        axis_label_font_family: Optional[Union[Array[String], List[str]]],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set an initial axis label font family setting.

        Parameters
        ----------
        axis_label_font_family : Optional[Union[Array[String], List[str]]]
            An axis label font family setting.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if axis_label_font_family is not None and not isinstance(
            axis_label_font_family, Array
        ):
            axis_label_font_family_: Optional[Array[String]] = Array(
                [String(font_name) for font_name in axis_label_font_family],  # type: ignore[union-attr] # noqa
                variable_name_suffix=variable_name_suffix,
            )
        else:
            axis_label_font_family_ = axis_label_font_family
        self._axis_label_font_family = axis_label_font_family_



"""The mix-in class implementation for the `SvgText`'s `_set_align` method.
"""

from typing_extensions import final

from apysc._display.svg_text_align_mixin import SvgTextAlign
from apysc._html.debug_mode import add_debug_info_setting


class SvgTextSetAlignMixIn:
    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_align(self, *, align: SvgTextAlign) -> None:
        """
        Set a text-align setting.

        Parameters
        ----------
        align : SvgTextAlign
            A text-align setting.
        """
        from apysc._display.svg_text_align_mixin import SvgTextAlignMixIn

        if not isinstance(self, SvgTextAlignMixIn):
            raise TypeError(
                f"This method is only supported an {SvgTextAlignMixIn.__name__} "
                f"instance: {type(self).__name__}"
            )

        self.align = align



"""The class implementation for the outlined `update` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialUpdateOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `update` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M11,8v5l4.25,2.52l0.77-1.28l-3.52-2.09V8H11z M21,10V3l-2.64,2.64C16.74,4.01,14.49,3,12,3c-4.97,0-9,4.03-9,9 s4.03,9,9,9s9-4.03,9-9h-2c0,3.86-3.14,7-7,7s-7-3.14-7-7s3.14-7,7-7c1.93,0,3.68,0.79,4.95,2.05L14,10H21z"/></g></g></svg>'  # noqa



"""The class implementation for the `block_flipped` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBlockFlippedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `block_flipped` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><path d="M22,12c0-5.5-4.5-10-10-10S2,6.5,2,12s4.5,10,10,10S22,17.5,22,12z M5.7,7.1l11.2,11.2c-1.3,1.1-3,1.7-4.9,1.7 c-4.4,0-8-3.6-8-8C4,10.1,4.6,8.4,5.7,7.1z M20,12c0,1.9-0.6,3.6-1.7,4.9L7.1,5.7C8.4,4.6,10.1,4,12,4C16.4,4,20,7.6,20,12z"/></g></svg>'  # noqa
